//  Diffusion Client Library for iOS, tvOS and OS X / macOS
//
//  Copyright (c) 2015, 2017 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;

@class PTDiffusionRecordMetadata;
@class PTDiffusionField;

NS_ASSUME_NONNULL_BEGIN

/**
 @brief A record is a collection of fields.

 A record object is immutable.

 @deprecated Since 6.0. Use PTDiffusionRecordV2 or PTDiffusionJSON.

 @since 5.6
 */
__deprecated_msg("Will be removed in a future release.")
@interface PTDiffusionRecord : NSObject <NSCopying>

+(instancetype)new NS_UNAVAILABLE;

-(instancetype)init NS_UNAVAILABLE;

/**
 The data defining the receiver.
 This returns the raw underlying bytes. It has not been transformed in any way.

 @since 5.6
 */
@property(nonatomic, readonly, copy) NSData* data;

/**
 The metadata defining the transformation applied to the receiver's data to produce the fields.

 @since 5.6
 */
@property(nonatomic, readonly, copy, nullable) PTDiffusionRecordMetadata* metadata;

/**
 Returns fields transformed by a PTDiffusionFieldMetadata object with the given name.

 If no fields were found with the given name or metadata is `nil` then this method will return
 an empty array.
 
 @param name The name to find when examining metadata to select fields.

 @exception NSInvalidArgumentException If the given name is `nil`.

 @since 5.6
 */
-(NSArray<PTDiffusionField *> *)fieldsWithName:(NSString *)name;

/**
 Returns the first field object transformed by a PTDiffusionFieldMetadata object with the given name.

 If no fields were found with the given name or metadata is `nil` then this method will return `nil`.

 @param name The name to find when examining metadata to select field.

 @exception NSInvalidArgumentException If the given name is `nil`.

 @since 5.6
 */
-(nullable PTDiffusionField *)fieldWithName:(NSString *)name;

/**
 Returns all fields encapsulated by the receiver. This array may be empty.

 @since 5.6
 */
@property(nonatomic, readonly) NSArray<PTDiffusionField *> * fields;

/**
 Compares the receiver to the given record.

 @param record The record object with which to compare the receiver.

 @return `YES` if the record is equal to the receiver, otherwise `NO`.

 @since 5.6
 */
-(BOOL)isEqualToRecord:(nullable PTDiffusionRecord *)record;

@end

NS_ASSUME_NONNULL_END
