//  Diffusion Client Library for iOS, tvOS and OS X / macOS
//
//  Copyright (c) 2015, 2017 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;
#import <Diffusion/PTDiffusionSendDeliveryPriority.h>

NS_ASSUME_NONNULL_BEGIN

/**
 @brief Send options are associated with sending content.
 
 @see PTDiffusionMessagingFeature

 @since 5.6
 */
@interface PTDiffusionSendOptions : NSObject <NSCopying, NSMutableCopying>

/**
 Returns a send options object initialized with empty headers and a Normal delivery priority.
 
 @since 5.6
 */
-(instancetype)init;

/**
 Returns a send options object initialized with empty headers and the given
 priority.

 @param priority The delivery priority to send with.

 @since 6.0
 */
-(instancetype)initWithPriority:(PTDiffusionSendDeliveryPriority)priority;

/**
 Returns a send options object initialized with the given headers and priority.

 @param headers The headers to send. This array may be empty.
 
 @param priority The delivery priority to send with.

 @deprecated Since 6.0. The use of headers on messages will be removed in a
 future release.

 @since 5.6
 */
-(instancetype)initWithHeaders:(NSArray<NSString *> *)headers
                      priority:(PTDiffusionSendDeliveryPriority)priority
__deprecated_msg("Will be removed in a future release.");

/**
 Returns a send options object with the given headers and priority.

 @param headers The headers to send. This array may be empty.

 @param priority The delivery priority to send with.

 @deprecated Since 6.0. The use of headers on messages will be removed in a
 future release.

 @since 5.6
 */
+(instancetype)sendOptionsWithHeaders:(NSArray<NSString *> *)headers
                             priority:(PTDiffusionSendDeliveryPriority)priority
__deprecated_msg("Will be removed in a future release.");

/**
 Returns a send options object with the given headers and a Normal delivery priority.

 @param headers The headers to send. This array may be empty.

 @deprecated Since 6.0. The use of headers on messages will be removed in a
 future release.

 @since 5.6
 */
+(instancetype)sendOptionsWithHeaders:(NSArray<NSString *> *)headers
__deprecated_msg("Will be removed in a future release.");

/**
 Returns a send options object with the given priority and empty headers.

 @param priority The delivery priority to send with.

 @since 5.6
 */
+(instancetype)sendOptionsWithPriority:(PTDiffusionSendDeliveryPriority)priority;

/**
 The headers to send.
 If there are no headers then this array may be empty.

 @deprecated Since 6.0. The use of headers on messages will be removed in a
 future release.

 @since 5.6
 */
@property(nonatomic, readonly, copy) NSArray<NSString *> * headers
__deprecated_msg("Will be removed in a future release.");

/**
 The delivery priority to send with.

 @since 5.6
 */
@property(nonatomic, readonly) PTDiffusionSendDeliveryPriority priority;

/**
 Compares the receiver to the given send options.

 @param sendOptions The send options object with which to compare the receiver.

 @return `YES` if the send options is equal to the receiver, otherwise `NO`.

 @since 5.6
 */
-(BOOL)isEqualToSendOptions:(nullable PTDiffusionSendOptions *)sendOptions;

@end

/**
 @brief Mutable send options can be modified prior to being associated with sending content.

 @since 5.6
 */
@interface PTDiffusionMutableSendOptions : PTDiffusionSendOptions

/**
 The headers to send.
 If there are no headers then this array may be empty.

 @deprecated Since 6.0. The use of headers on messages will be removed in a
 future release.

 @since 5.6
 */
@property(nonatomic, readwrite, copy) NSArray<NSString *> * headers
__deprecated_msg("Will be removed in a future release.");

/**
 The delivery priority to send with.

 @since 5.6
 */
@property(nonatomic, readwrite) PTDiffusionSendDeliveryPriority priority;

@end

NS_ASSUME_NONNULL_END
